/*
  File Name  : ckEditorApi.js
  Description: CKEditor wrapper functions.

  Licensed Materials - Property of IBM - some unique copyrights (i.e., the ability 
  to copy, modify, distribute) may be licensed in certain circumstances 
  (e.g., sample/template code intended to be used by customers to build upon and 
  distribute as their own products).

  DOORS - %full_filespec: ckeditorApi.js~16:javascript:ERS#1 %  %date_created: Mon Jan  7 13:50:23 2013 %

 (c) Copyright IBM Corporation 2011. All Rights Reserved.

  U.S. Government Users Restricted Rights: Use, duplication or disclosure restricted
  by GSA ADP Schedule Contract with IBM Corp.
*/

// JavaScript API for DOORS client core code calls.

currentEditorId = "editor1";

//*********************************************************************************
// DESCRIPTION: KeyDown callback for CKEditor, 
//              this allows a selection of keydown events occuring within the CKEditor 
//              to be fired back to the Rich Client container window,
//              this gives the chance to the Rich Client to process specific shortcuts.
CKEDITOR.on('instanceCreated', function (e) {
    e.editor.on('contentDom', function () {
        var currentEditor = CKEDITOR.instances.editor1
        e.editor.document.on('click', clickHandler);
    });
});

// Make the div contents editable by replacing it with an editor instance
// in the display.
function showEditor(toolbarString)
{
    var editor = currentEditor;
    if (!editor)
    {
        // Use the default toolbar if none is specified.
        if (toolbarString.length == 0)
        {
            toolbarString = defaultToolbarString;
        }
        // Replace the <div id="editor1"> with an CKEditor
        // instance, using RRC style configurations
        var theToolbar = toolbarArray(toolbarString);
        
        currentEditor = editor = CKEDITOR.replace("editor1", {
            skin: (CKEDITOR.version == "3.4.2") ? "v2" : "rational",
        	toolbar: theToolbar,
        	height: '600px',
        	removePlugins: "elementspath,sametimeemoticons,customdialogs,lotustoolbars,urllink,lotusstatusmessage,lotusbidi,doclink",
        	coreStyles_bold: {
        		element: "b" // use a <b> instead of a <strong>
        	},
        	coreStyles_italic: {
        		element: "i" // use an <i> instead of an <em>
        	},
        	coreStyles_strike: {
        		element: "del" // use <del> instead of <strike>
        	},
        	enableTabKeyTools: true,
        	entities: false, // html entities are not valid xhml
        	entities_latin: false,
        	entities_greek: false
        });

        editor.on('instanceReady', function (e)
        {

            var editor = CKEDITOR.instances.editor1;
            editor.document.$.body.style.margin = "0px";

            // Resize the editor to match the browser window size.
            editor.resize(document.body.offsetWidth, document.body.offsetHeight);

            // Output filter rules..
            editor.dataProcessor.htmlFilter.addRules(htmlFilterRules);
        });

        editor.on('paste', pasteHandler);
    }    
}

// Hide the editor and display the read-only div.
function hideEditor(updateDiv)
{
    var editor = currentEditor;
    if (editor)
    {
        // Initial call seems to fail first time for a given htmlEdit: 
        // try a 2nd time if so: seems to succeed.
        try
        {
            editor.destroy(!updateDiv);
        }
        catch(err)
        {
            try
            {
                editor.destroy(!updateDiv);
            }
            catch(err2)
            {
                // Do nothing.
            }
        }
        currentEditor = false;
    }
}

/* end */