/*
  File Name  : ckEditorApi.js
  Description: CKEditor wrapper functions.

  Licensed Materials - Property of IBM - some unique copyrights (i.e., the ability 
  to copy, modify, distribute) may be licensed in certain circumstances 
  (e.g., sample/template code intended to be used by customers to build upon and 
  distribute as their own products).

  DOORS - %full_filespec: ckeditorApiCommon.js~2:javascript:ERS#1 %  %date_created: Wed Jan  9 09:32:33 2013 %

 (c) Copyright IBM Corporation 2011. All Rights Reserved.

  U.S. Government Users Restricted Rights: Use, duplication or disclosure restricted
  by GSA ADP Schedule Contract with IBM Corp.
*/

// JavaScript API for DOORS client core code calls.

/*************************************************************************************
*                                                                                    *
* NOTE: THE VARIABLES AND FUNCTIONS IN THIS FILE ARE USED BY CODE IN MULTIPLE FILES  *
*        (specifically ckeditorApi.js and gridCKEditorApi.js)                        *
*        SO ANY CHANGES MUST BE COMPATIBLE WITH THEIR USAGE IN ALL OF THOSE FILES.   *
*                                                                                    *
**************************************************************************************/

// Keep latest editor contents, in case it needs to be restored after an invalid paste operation
var previousContent = ""

// Keep latest contents that was pasted
var contentsToPaste = ""

var dummyDivId = "dummyDiv"
var errorIcon = "/web/com.ibm.rdm.web/common/images/error.gif"

var currentEditor = false;
var currentEditorId = "";

var busyScripts = 0; // == 0 if nothing important is going on, will be increased by setScriptBusy(), and decreased by setSriptIdle()

var defaultToolbarString = "Format,Font,FontSize,-," +
"Bold,Italic,Underline,Strike,Subscript,Superscript,-," +
"Cut,Copy,Paste,PasteText,PasteFromWord,-,Undo,Redo,-,Find:" +
"TextColor,BGColor,-," +
"JustifyLeft,JustifyCenter,JustifyRight,JustifyBlock,Indent,Outdent,-," +
"BulletedList,NumberedList,Table,HorizontalRule,-," +
"Link,Unlink";

//*****************************************************
function splitAndTrim(listString,separator)
// DESCRIPTION:
// Function splitting the supplied listString using a separator,
// and removing whitespaces from start and end of each entry 
// in the returned array.
{
    var firstResult = listString.split(separator);
    var trimmed = [];
    for (i in firstResult)
    {
        trimmed.push(CKEDITOR.tools.trim(firstResult[i])); 
    }
    return trimmed;
}

//*****************************************************
function toolbarArray(toolbarString)
//
// DESCRIPTION: From the supplied string composed of button names
//              separated by colons (for sections) and commas,
//              builds a toolbar array suitable for use by 
//              CKEDITOR.replace() to create a toolbar.
{
    var theToolbar = [];
    if (toolbarString.length != 0)
    {
        var toolbarStrings = splitAndTrim(toolbarString,":");
        for (i in toolbarStrings)
        {
            var barString = toolbarStrings[i];
            theToolbar.push(splitAndTrim(barString,","));
        }
    }
    return theToolbar;
}

//*************************************************************
function removeIdAttributeFromPTags(html)
// DESCRIPTION: Remove all id attribute from all P tags found in html contents
// RETURNS: New contents with P tags without id attribute
{
    //evt.data.html = evt.data.html.replace(/<(P)[^>]+>/ig,'<$1>'); // This is good but all attributes are remove, only <p> is kept
    var wrapper = document.createElement('div');
    wrapper.innerHTML = html;
    var pElements = wrapper.getElementsByTagName('p');

    for (var i = 0; i < pElements.length; i++) 
    {
        pElements[i].removeAttribute('id');
        pElements[i].removeAttribute('ID');
    }

    var newHtml = wrapper.innerHTML;
    return newHtml;
}

//*************************************************************
function isScriptIdle()
// DESCRIPTION: Gets the idle state of the java script
// RETURNS: busyScripts, which is > 0 if the script is busy doing something important such as:
//          - ckeditor construction / destruction
{
    return (busyScripts == 0);
}

//*************************************************************
function setScriptBusy()
// DESCRIPTION: Increase busyScripts by one, this is a clear indication that the script is busy !
//              For safety with events, and to avoid script errors, all important methods should call setScriptBusy() on enter, 
//              and call setScriptIdle() on exit.
{
    busyScripts++;
}

//*************************************************************
function setScriptIdle()
// DESCRIPTION: Decrease busyScripts by one, if back to zero then this is a clear indication that the script is idle !
//              For safety with events, and to avoid script errors, all important methods should call setScriptBusy() on enter, 
//              and call setScriptIdle() on exit.
{
    if (busyScripts > 0)
    {
        busyScripts--;
    }
}

//*********************************************************************************
// DESCRIPTION: Gets the dialog definition, checks for link dialog 
//              removes target and advanced tabs for the link dialog
CKEDITOR.on('dialogDefinition', function (ev) {
    var dialogName = ev.data.name;
    var dialogDefinition = ev.data.definition;
    if (dialogName == 'link') {
        dialogDefinition.removeContents('target');
        dialogDefinition.removeContents('advanced');
    }
});

//*************************************************************
function restorePreviousContents()
// DESCRIPTION: Restore previous content to current editor.
{
    if (currentEditor)
    {
        setContents(previousContent);
    }
}

//*************************************************************
function getContentsToPaste()
// DESCRIPTION: Gets contents to paste.
// RETURNS: html contents.
{
    return contentsToPaste;
}
        
//*************************************************************
function getContents()
// DESCRIPTION: Gets the current contents of the current editor.
// RETURNS: html contents.  Empty string on failure.
{    
    var contents = "";
    if (currentEditor)
    {
        // Replace any &nbsp; instances: not valid xhtml
        contents = currentEditor.getData().replace(/&nbsp;/g, "&#160;");       
    }
    else
    {
        var theDiv = document.getElementById(currentEditorId);
        
        if (!theDiv)
        {
            alert("getContents(): No editor! " + currentEditorId);
        }
        else
        {
            contents = theDiv.innerHTML;
        }
    }

    return contents;
}

//*************************************************************
function setContents(contents)
// DESCRIPTION: Sets the current contents of the current editor or newEditDivId.
{  
    setScriptBusy();    
    if (currentEditor)
    {
        // Replace any &nbsp; instances: not valid xhtml
        currentEditor.setData(contents);
    }
    else
    {
        var theDiv = document.getElementById(currentEditorId);
            
        if (!theDiv)
        {
            alert("getContents(): No editor! " + currentEditorId);
        }
        else
        {
            theDiv.innerHTML = contents;
        }
    }
    setScriptIdle();
}

//*************************************************************
function getModified()
// DESCRIPTION: Checks whether the contents of the current editor 
//              have been modified by the user.
// RETURNS: true if the editor exists and has been edited, else false.
{            
    setScriptBusy();  
        
    // Exit if the dummy div contains the current editor,
    // the dummy div can't be edited !  
    var modified = currentEditor 
        && currentEditor.name != dummyDivId
        && currentEditor.isReady
        && currentEditor.checkDirty();
    
    setScriptIdle();
    
    return modified;
}


function resetModified()
// DESCRIPTION: Reset the modified flag of the editor to FALSE
// RETURNS: void
{
    if (currentEditor) {
        currentEditor.resetDirty();
    }
}

//*******************************************************
function clickHandler(e) // e : event
//
// DESCRIPTION: Callback to handle click events in CKEditor instances.
{
    if (!isScriptIdle())
    {
        return;
    }
    
    if (currentEditor && currentEditor.name != dummyDivId)
    {
        var event = e.data.$;
        var linkTarget = e.data.getTarget();
        if (linkTarget.is('a') && linkTarget.$.href && (linkTarget.$.protocol.indexOf('http') == 0))
        {
            if (event.ctrlKey == 1)
            {
                window.open(linkTarget.$.href);
            }
            currentEditor.document.$.body.contentEditable = true;
        }
        else if (linkTarget.is('img') && linkTarget.$.href && linkTarget.$.href.search(errorIcon) >= 0) 
        {
            window.open(linkTarget.$.href);
            e.data.preventDefault();
        }
    }
}

var htmlFilterRules =
{
    attributeNames:
    [
    // Discard the following attributes.
        [(/^xml:lang$/), ''],
        [(/^_fck_bookmark$/), ''],
        [(/^_zipidx$/), ''],
        [(/^unselectable$/), ''],
        [(/^contentEditable$/), ''],
    ],

    elements:
    {
        // Replace <font> with inert <span>
        font: function (element)
        {
            element.name = 'span';
            element.attributes = [];
        },

        // Discard any empty <tr> tags
        tr: function (element)
        {
            if (element.children.length == 0)
            {
                return false;
            }
        }
    }
};

//**********************************************************
function pasteHandler(evt)
//
// DESCRIPTION: paste event handler for editor instances.
//              Strips out paragraph id values before copying contents
//              into editor, and sends a 'paste:' navigate signal to the 
//              browser control.
{
    var editor = evt.editor;
    previousContent = evt.editor.getData(); // Keep previous contents before paste
    evt.data.html = removeIdAttributeFromPTags(evt.data.html); // Remove all id attributes for p tags
    contentsToPaste = evt.data.html; // Keep contents that was pasted         
    setTimeout("window.location.replace('paste:')", 300); // Delegate validation of pasted contents to client container
} 
/* end */