/*
  File Name  : htmlGrid.js

  Description: General functions for the DOORS "HTML grid" control.

  Licensed Materials - Property of IBM - some unique copyrights (i.e., the ability 
  to copy, modify, distribute) may be licensed in certain circumstances 
  (e.g., sample/template code intended to be used by customers to build upon and 
  distribute as their own products).

  DOORS - %full_filespec: htmlGrid.js~1.1.12.1.2.2.4:javascript:ERS#1 %  %date_created: Fri Jan  4 16:10:17 2013 %

 (c) Copyright IBM Corporation 2012. All Rights Reserved.

  U.S. Government Users Restricted Rights: Use, duplication or disclosure restricted
  by GSA ADP Schedule Contract with IBM Corp.

*/

var lastTimeout = false;
var scrollDiv = false;
var headingsDiv;
var toolBarHeight = 0;

// Initially off.  First manual call to resizeScrollDiv turns on window.onresize event driven resizing
// Can be turned off temporarily whilst toolbar, ckeditor instances are shuffled around.
var enableAutoScrollResize = false; 

function doResizeScrollDiv()
// DESCRIPTION: Resize the vertical scroll bar (div) to accommodate the whole table.
{
    if (!scrollDiv || !enableAutoScrollResize)      
    {
        return;
    }

    var remainingHeight = window.document.documentElement.offsetHeight // IE in or out of quirks mode
                        - window.document.getElementById('gridHeadings').offsetHeight
                        - 4;    // fudge factor that prevent toobar shudder on changing row (and resizing "recursion")
    
    if (currentEditor)
    {
        if (toolBarHeight == 0) // Can be cached as long as first call is triggered manually (rather than window.onResize)
        {
            var theToolbar = window.document.getElementById('sharedToolbar');
            toolBarHeight = theToolbar.offsetHeight;
        }
        remainingHeight -= toolBarHeight;
    }

    if (remainingHeight > 0)
    {
        //alert("resize " + currentEditor.name + ": " + window.document.documentElement.offsetHeight + " - hdg" + window.document.getElementById('gridHeadings').offsetHeight + " - tb" + toolBarHeight + " -2 = " + remainingHeight);
        scrollDiv.style.height = remainingHeight + "px";
    }
}

function resizeScrollDiv()
{
    enableAutoScrollResize = true;      // Manual call, always executes resize, and (re-)turns on auto-resize therafter
    doResizeScrollDiv();
}

function resizeScrollDivOnWindowResize()
{
    doResizeScrollDiv()
    return true;
}

function findPos(obj) 
// DESCRIPTION: Finds y value of given object.
// RETURNS: the y value of given object.
{
    var curtop = 0;
    if (obj.offsetParent) 
    {
        do
        {
            curtop += obj.offsetTop;
        }
        while (obj = obj.offsetParent);

        return [curtop];
    }
}

// Store each scrollOffset for comparison with next scroll offset,
// to determine scrolling direction.
var scrollOffset = 0;

function scrollDone()
// DESCRIPTION: Called to react to a completed scroll action - generates a
//              navigate event according to the direction of scrolling.
// The new location strings should be intercepted by WebBrowserEvents to
// refresh the display and veto the navigation.
{
    // Generate a navigate request depending on scrolling direction.
    if (scrollDiv.scrollTop > scrollOffset)
    {
        window.location.replace("div:scrollDown");
    }
    else
    {
        window.location.replace("div:scrollUp");
    }
    // Store new offset for comparison in next scroll event.
    scrollOffset = scrollDiv.scrollTop;
}

function scrollHandler()
// DESCRIPTION: Callback for scroll activity 
//              - tries to navigate to "div:scrollUp" or "div:scrollDown"
//                if it isn't called again before 300ms.
{
    // Handle horizontal scrolling - keep columns headings aligned.
    headingsDiv.scrollLeft = scrollDiv.scrollLeft;

    // Handle vertical scrolling
    if (scrollDiv.scrollTop != scrollOffset)
    {
        if (lastTimeout)
        {
            clearTimeout(lastTimeout);
        }
        lastTimeout = setTimeout("scrollDone()", 300);
    }
} 

//***************************************************************
function delayedScroll(position)
//
// DESCRIPTION: Function to allow navigation on module open to work
//              for artifacts at the end of the module.  Without this,
//              the module never scrolls quite to the end of the grid
//              if scrollTop is set directly by the document complete
//              callback code.
{
    setTimeout("scrollDiv.scrollTop = " + position, 1);
}

//*****************************************************************
function validateNumber(elt, value, numValue, minValue, maxValue)
//
// DESCRIPTION: Updates the style of elt depending on the values
//               supplied - numValue must be a number and in string
//              form equal to value.  Applies minValue and maxValue
//              limits if they're numbers.
{
    // 'e' condition below disallows scientific notation, in line with web client.
    if (value.length > 0 && 
            (isNaN(numValue) || 
            numValue != value || 
            value.toLowerCase().indexOf('e') >= 0 || 
            (!isNaN(minValue) && numValue < minValue) || 
            (!isNaN(maxValue) && numValue > maxValue)))
    {
        // Invalid value - set background pink
        elt.style.backgroundColor = "#FF9999";
    }
    else
    {
        // Valid value - update goodvalue attribute and set background white
        elt.style.backgroundColor = "white";
        elt.setAttribute('goodvalue', value);
    }
}

// The server only accepts integers between -(2**31) and (2**31)-1
var maxAcceptedInt = Math.pow(2,31)

//************************************************
function validateInt(elt)
//
// DESCRIPTION: Checks the value of the specified <input> element for valid
//              integer format.  Changes the element border to red if invalid.
//              Else, copies latest value to 'goodValue' attribute.
{
    var value = elt.getAttribute('value');
    var minString = elt.getAttribute('minvalue');
    var maxString = elt.getAttribute('maxvalue');
    
    var intValue = parseInt(value, 10);
    var minValue = parseInt(minString, 10);
    var maxValue = parseInt(maxString, 10);
    
    // Impose server limits for integer values
    if (isNaN(minValue))
    {
        minValue = 0 - maxAcceptedInt
    }
    if (isNaN(maxValue))
    {
        maxValue = maxAcceptedInt - 1
    }
    
    validateNumber(elt, value, intValue, minValue, maxValue);
}

// Web client only accepts values in this range
var maxAcceptedFloat = 9 * Math.pow(10,15)

//************************************************
function validateFloat(elt)
//
// DESCRIPTION: Checks the value of the specified <input> element for valid
//              float format.  Changes the element border to red if invalid.
//              Else, copies latest value to 'goodValue' attribute.
{
    var value = elt.getAttribute('value');
    var minString = elt.getAttribute('minvalue');
    var maxString = elt.getAttribute('maxvalue');
    
    var floatValue = parseFloat(value);
    var minValue = parseFloat(minString);
    var maxValue = parseFloat(maxString);

    // Impose web client limits for float values
    if (isNaN(minValue))
    {
        minValue = 0 - maxAcceptedFloat
    }
    if (isNaN(maxValue))
    {
        maxValue = maxAcceptedFloat
    }
    validateNumber(elt, value, floatValue, minValue, maxValue);
}

//************************************************
function validateDurationInput(elt)
//
// DESCRIPTION: Checks the value of the specified <input> element for valid
//              integer format.  Changes the element border to red if invalid.
//              Else, copies latest value to 'goodValue' attribute.
{
    var maxString;
    var maxValue;
    var mySelect;
    var newElt;
    var eltId = elt.getAttribute('id');
    if (eltId == "durationinput")
    {
        // The element is the numeric input, reset the maxValue if 'Weeks' is selected.
        var selectElt = document.getElementById('duration');
        mySelect = selectElt.options[selectElt.selectedIndex].text;
        if (mySelect == "Weeks")
        {
            maxString = elt.getAttribute('maxvalue');
            maxValue = parseInt(maxString, 10);
            if (maxValue == (maxAcceptedInt-1))
            {
                maxValue = maxValue/7;
                maxValue = Math.floor(maxValue);
                elt.setAttribute('maxvalue', maxValue);
            }
        }
        else
        {
            elt.setAttribute('maxvalue', (maxAcceptedInt-1));
        }
        newElt = elt;
    }
    else if (eltId = "duration")
    {
        // The element is the drop down selection, reset the maxValue if 'Weeks' is selected.
        var inputElt = document.getElementById('durationinput');
        mySelect = elt.options[elt.selectedIndex].text;
        if (mySelect == "Weeks")
        {
            maxString = inputElt.getAttribute('maxvalue');
            maxValue = parseInt(maxString, 10);
            if (maxValue == (maxAcceptedInt-1))
            {
                maxValue = maxValue/7;
                maxValue = Math.floor(maxValue);
                inputElt.setAttribute('maxvalue', maxValue);
            }
        }
        else
        {
            inputElt.setAttribute('maxvalue', (maxAcceptedInt-1));
        }
        newElt = inputElt;
    }
    else
    {
        newElt = elt;
    }

    // Call validateInt
    validateInt(newElt);

    // Copy the latest drop down selection to 'goodselect' attribute.
    if (newElt.style.backgroundColor == "white")
    {
        newElt.setAttribute('goodselect', mySelect);
    }
}

//************************************************
function handleNavigationKeys(elt)
//
// DESCRIPTION: Select the hyperlink within the column cell using up and down arrow key.
{
    var keynum = window.event.keyCode;
    if (keynum == 27) // Esc key
    {
        // Remove focus from the element when pressing Esc
        elt.blur();
    }
    else if (!window.event.altKey && (keynum == 38 || keynum == 40 || keynum == 9)) // Up or Down arrow or Tab
    {
        var tagname = elt.tagName;
        var parentElt = elt.parentNode;
        var clsName = parentElt.className;
        if (clsName == "linkDiv")
        {
            parentElt = parentElt.parentNode;
        }
        var elts = parentElt.getElementsByTagName(tagname);
        var newElt;
        if (keynum == 9) // Tab - reset focus on the first link element
        {
            newElt = elts.item(0);
        }
        else
        {
            var i, newIndex;
            for (i = 0; i < elts.length; i++)
            {
                if (elts.item(i) == elt)
                {
                    break;
                }
            }
            if (keynum == 38) // Up arrow
            {
                newIndex = i - 1;
            }
            else if (keynum == 40) // Down arrow
            {
                newIndex = i + 1;
            }
            if (newIndex < 0 || newIndex > (elts.length-1))
            {
                return;
            }
            newElt = elts.item(newIndex);
        }
        newElt.focus();
    }
}

/*************************************************************
function showHTML()
// DESCRIPTION: Debug code to display the contents of the grid in a 
//              textarea below the grid.
{
    var theGrid = window.document.getElementById('theGrid');
    var htmltext = window.document.getElementById('debugtext');
    if (!htmltext)
    {
        var contents = scrollDiv.innerHTML;
        scrollDiv.innerHTML = scrollDiv.innerHTML + '<textarea id="debugtext" cols=100 rows=20></textarea>';
        htmltext = window.document.getElementById('debugtext');
        htmltext.value = contents;
    }
    else
    {
        if (confirm('Copy html to clipboard?'))
        {
            var Copied = htmltext.createTextRange();
            Copied.execCommand("Copy");
        }
        htmltext.parentNode.removeChild(htmltext);
    }
}
*/

function initDivs()
// DESCRIPTION: Initialises the grid div variables, and sets the scroll handler
//              callback on the main scrolling grid div and the resize handler
//              on the window.  The variables must be
//              initialised first, because the handlers use them.
{
    scrollDiv = window.document.getElementById('scrollDiv');
    headingsDiv = window.document.getElementById('headingsDiv');
    scrollDiv.onscroll = scrollHandler;
    window.onresize = resizeScrollDivOnWindowResize;

    // Signal completion to core code - WebBrowserEvents should prevent navigation.
    window.location.replace('win:loaded');
    // headingsDiv.onclick = showHTML; // Debug only
}

// Window behaviours    
window.onload = initDivs;

/* End */
